import { parser } from 'lezer-glsl';
import { LRLanguage, indentNodeProp, foldNodeProp, continuedIndent, delimitedIndent, foldInside, LanguageSupport } from '@codemirror/language';

/**
A language provider based on the [Lezer GLSL
parser](https://github.com/schimd-daniel/lezer-glsl), extended with
highlighting and indentation information.
*/
const glslLanguage = /*@__PURE__*/LRLanguage.define({
    name: "glsl",
    parser: /*@__PURE__*/parser.configure({
        props: [
            /*@__PURE__*/indentNodeProp.add({
                IfStatement: /*@__PURE__*/continuedIndent({ except: /^\s*({|else\b)/ }),
                CaseStatement: (context) => context.baseIndent + context.unit,
                BlockComment: () => null,
                CompoundStatement: /*@__PURE__*/delimitedIndent({ closing: "}" }),
                Statement: /*@__PURE__*/continuedIndent({ except: /^{/ }),
            }),
            /*@__PURE__*/foldNodeProp.add({
                "StructDeclarationList CompoundStatement": foldInside,
                BlockComment(tree) {
                    return { from: tree.from + 2, to: tree.to - 2 };
                },
            }),
        ],
    }),
    languageData: {
        commentTokens: { line: "//", block: { open: "/*", close: "*/" } },
        indentOnInput: /^\s*(?:case |default:|\{|\})$/,
        closeBrackets: {
            stringPrefixes: ["L", "u", "U", "u8", "LR", "UR", "uR", "u8R", "R"],
        },
    },
});
/**
Language support for GLSL.
*/
function glsl() {
    return new LanguageSupport(glslLanguage);
}

export { glsl, glslLanguage };
